/*
 * $Id: penc.c,v 0.5 2000/02/29 porokh Exp $
 *	v 0.4 1999/02/13 porokh
 *	v 0.3 1998/08/22 porokh
 *	v 0.2 1997/08/12 porokh
 *	v 0.1 1997/08/09 porokh
 *
 ***************************************************************************
 *
 * PEnc - Cyrillic recoding utility (with CR/LF <-> LF handling)
 * (c) 1997,2000 Andrew Porokhnyak
 *
 * To make binary:
 *	cc -o penc penc.c	(GNU C)
 *	tcc penc.c		(Turbo C)
 *	wcl penc.c		(Watcom C)
 */

#include "portab.h"
#ifdef __unix__
    #include <unistd.h>
#else
    #include <io.h>
#endif
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define DEB

/*** preprocessor macros **************************************************/

/* program run modes */
#define PGM_STOP        0x7fff
#define PGM_STDIO       0

/* error codes */
#define ERR_OPEN_INP    11
#define ERR_OPEN_OUT    12
#define ERR_NO_R_INFO   13

#define ERR_CODE_OPEN   21
#define ERR_CODE_READ   22
#define ERR_CODE_SEEK   23


/*** global variables *****************************************************/
const char prod_name[]="PEnc";
const char prod_ver1=0;
const char prod_ver2=5;
const char prod_ver3=0;
const char prod_date[]="20000229";
const char prod_copr[]="(c) 1997,2000 Andrew Porokhnyak";
const char prod_help[]=
"This is a stream encoding program, operated with cyrillic encodings\n"
"(DOS, UNIX, Win) and different new-line separators (DOS, UNIX)\n\n"
"Usage:\n"
"\tpenc encoding_optn [newline_optn] [-o output_file] [input_file]\n\n"
"Default input and output is stdin and stdout\n"
"newline_optn:\n\t-td2u   CR/LF to LF\t\t-tu2d   LF to CR/LF\n"
"encoding_optn:"
;

#include "penc_cod.h"

/*** function prototypes **************************************************/

int parse_params(int, char **, char *, char *, unsigned char *, unsigned
    char *);
int open_files(FILE **, char *, FILE **, char *);
int get_codetable(char *, unsigned char, unsigned char *);


/***************************************************************************
 *   main()
 *
 *	1997/08/12, Porokh
 *	1998/08/22, Porokh
 *	2000/02/29, Porokh
 **************************************************************************/
int main(int argc, char **argv) {
    unsigned int _rm=0;         /* run-mode, bit-mapped */
    FILE *_inp, *_out;          /* input and output files */
    char finp_name[256]="";     /* inp file name */
    char fout_name[256]="";     /* out file name */
    unsigned char *code_tbl=NULL;
    char code_no=99;    	/* code table number */
    char text_no=0;             /* line delimiter translation number: */
                                /* 1 CR/LF > LF, 2 LF > CR/LF */
                                /* 3 CR/LF > CR, 4 CR > CR/LF */
    int c;
    int retc;

    _rm=parse_params(argc, argv, finp_name, fout_name, &code_no, &text_no);
    if (_rm == PGM_STOP) return 0;
    code_tbl=(unsigned char *)CODES + code_no*128;

    _inp=stdin;
    _out=stdout;

    if ((retc=open_files(&_inp, finp_name, &_out, fout_name)) != 0) {
        printf("Error %i\n", retc);
        return retc;
    }

    while ((c=fgetc(_inp)) != EOF) {
        if (text_no) {
            if ((text_no == 1) && (c == 0xd)) {
                if ((c=fgetc(_inp)) != 10)
                    fputc(13, _out);
            }
            if ((text_no == 2) && (c == 10))
                fputc(13, _out);
        }
        if ((c < 128) || (code_no == 99))
            fputc(c, _out);
        else
            fputc((int)code_tbl[c-128], _out);
    }

    fclose(_inp);
    fclose(_out);

    return 0;
} /*----------------------------------------------------------------------*/


/***************************************************************************
 *   parse_params()
 *
 * Analyse command-line options
 *
 * Return: program run mode, see PGM_* for details
 * Change: *finp_name, *fout_name, code_no, text_no
 *
 *	1997/08/12, Porokh
 *	1998/08/22, Porokh
 *	2000/02/29, Porokh
 **************************************************************************/
int parse_params(
        int argc,
        char **argv,
        char *finp_name,
        char *fout_name,
        unsigned char *code_no,
        unsigned char *text_no
        ) {

    int retcode=0;              /* return code */
    int arg=1;                  /* currently parsed argument */
    int i;

    if ((argc==1)
	|| (strcmp(argv[1], "-?") == 0)
        || (strcmp(argv[1], "-h") == 0)
        || (strcmp(argv[1], "--help") == 0)) {
        fprintf(stdout, "\n\t%s v%d.%02d.%02d %s (%s) %s\n"
            "\tCompiled: %s by %s\n", prod_name, prod_ver1, prod_ver2,
            prod_ver3, TARGETOS, prod_date, prod_copr, __DATE__, MADEBY);
        fprintf(stdout, "\n%s\n%s", prod_help, CODE_DESC);
        return PGM_STOP;
    }
    
    /* Check for arguments in cmd line */
    while (arg<argc) {
        if (strcmp(argv[arg], "-td2u") == 0) {
            arg++;
            *text_no=1;
            continue;
        }
        if (strcmp(argv[arg], "-tu2d") == 0) {
            arg++;
            *text_no=2;
            continue;
        }
        if (strcmp(argv[arg], "-o") == 0) {
            arg++;
            if (arg>=argc) break;
            if (argv[arg][0]=='-') continue;
            strcpy(fout_name, argv[arg]);
            arg++;
            fprintf(stderr, "output file: %s\n", fout_name);
            continue;
        }
        /* check */
        for (i=0; i<CODE_CNTS; i++) {
            if (strcmp(argv[arg], code_key[i]) == 0) {
                arg++;
                *code_no=i;
                i=99;
                break;
            }
        }
        if (i==99)
            continue;
        /* */
        if (argv[arg][0] == '-') {
            fprintf(stderr, "parse_params: unknown param '%s'\n", argv[arg]);
            arg++;
            continue;
        }
        strcpy(finp_name, argv[arg]);
        fprintf(stderr, "input file: %s\n", finp_name);
        arg++;
    }

    return retcode;
} /*----------------------------------------------------------------------*/


/***************************************************************************
 *   open_files()
 *
 * Open input and output files (stdin and stdout by default)
 *
 * Return: 0 if no err, or err_code
 * Change: *_inp, *_out
 *
 *	1997/08/12, Porokh
 **************************************************************************/
int open_files(
        FILE **_inp,
        char *finp_name,
        FILE **_out,
        char *fout_name
        ) {

    if (*finp_name) {
        if ((*_inp=fopen(finp_name, "r")) == NULL)
            return ERR_OPEN_INP;
    }
    if (*fout_name) {
        if ((*_out=fopen(fout_name, "a")) == NULL)
            return ERR_OPEN_OUT;
    }

    return 0;
} /*----------------------------------------------------------------------*/

/*** EOF penc.c ***********************************************************/
